searchNodes=[{"doc":"A set of functions to deal with analytical formulae. The typical way of using this module would be to call Formulae.compile/1 on the binary representing the string. iex | 1 ▶ f = Formulae . compile &quot;a + :math.sin(3.14 * div(b, 2)) - c&quot; % Formulae { ast : { :- , [ line : 1 ] , [ { :+ , [ line : 1 ] , [ { :a , [ line : 1 ] , nil } , { { :. , [ line : 1 ] , [ :math , :sin ] } , [ line : 1 ] , [ { :* , [ line : 1 ] , [ 3.14 , { :div , [ line : 1 ] , [ { :b , [ line : 1 ] , nil } , 2 ] } ] } ] } ] } , { :c , [ line : 1 ] , nil } ] } , eval : &amp; :&quot;Elixir.Formulae.a + :math.sin(3.14 * div(b, 2)) - c&quot; . eval / 1 , formula : &quot;a + :math.sin(3.14 * div(b, 2)) - c&quot; , module : :&quot;Elixir.Formulae.a + :math.sin(3.14 * div(b, 2)) - c&quot; , variables : [ :a , :b , :c ] } Now the formula is compiled and might be invoked by calling Formulae.eval/2 passing a formula and bindings. First call to eval/2 would lazily compile the module if needed. iex | 2 ▶ f . eval . ( a : 3 , b : 4 , c : 2 ) 0.9968146982068622 The formulae might be curried. iex | 3 ▶ Formulae . curry ( f , a : 3 , b : 4 ) % Formulae { ast : ... , eval : &amp; :&quot;Elixir.Formulae.3 + :math.sin(3.14 * div(4, 2)) - c&quot; . eval / 1 , formula : &quot;3 + :math.sin(3.14 * div(4, 2)) - c&quot; , module : :&quot;Elixir.Formulae.3 + :math.sin(3.14 * div(4, 2)) - c&quot; , variables : [ :c ] }","ref":"Formulae.html","title":"Formulae","type":"module"},{"doc":"Returns the binding this formula requires. Examples iex&gt; &quot;a &gt; 5&quot; |&gt; Formulae . bindings? ~w|a|a iex&gt; &quot;:math.sin(a / (3.14 * b)) &gt; c&quot; |&gt; Formulae . bindings? ~w|a b c|a iex&gt; &quot;a + b * 4 - :math.pow(c, 2) / d &gt; 1.0 * e&quot; |&gt; Formulae . bindings? ~w|a b c d e|a","ref":"Formulae.html#bindings?/2","title":"Formulae.bindings?/2","type":"function"},{"doc":"Revalidates the formula with bindings given. Returns true if the formula strictly evaluates to true , false otherwise. Compiles the formula before evaluation if needed.","ref":"Formulae.html#check/2","title":"Formulae.check/2","type":"function"},{"doc":"Generated clauses for n ∈ [1..42] to be used with dynamic number","ref":"Formulae.html#combinations/2","title":"Formulae.combinations/2","type":"function"},{"doc":"Compiles the formula into module. Examples: iex&gt; f = Formulae . compile ( &quot;rem(a, 5) - b == 0&quot; ) iex&gt; f . formula &quot;rem(a, 5) - b == 0&quot; iex&gt; f . variables [ :a , :b ] iex&gt; f . module :&quot;Elixir.Formulae.rem(a, 5) - b == 0&quot; iex&gt; f . module . eval ( a : 12 , b : 2 ) true iex&gt; f = Formulae . compile ( &quot;rem(a, 5) + b == a&quot; ) iex&gt; f . variables [ :a , :b ] iex&gt; f . eval . ( a : 7 , b : 5 ) true iex&gt; f . eval . ( a : 7 , b : 0 ) false","ref":"Formulae.html#compile/2","title":"Formulae.compile/2","type":"function"},{"doc":"Checks whether the formula was already compiled into module. Typically one does not need to call this function, since this check would be nevertheless transparently performed before the evaluation. Examples: iex&gt; Formulae . compiled? ( &quot;foo &gt; 42&quot; ) false iex&gt; Formulae . compile ( &quot;foo &gt; 42&quot; ) iex&gt; Formulae . compiled? ( &quot;foo &gt; 42&quot; ) true","ref":"Formulae.html#compiled?/2","title":"Formulae.compiled?/2","type":"function"},{"doc":"Curries the formula by substituting the known bindings into it. Example iex&gt; Formulae . curry ( &quot;(temp - foo * 4) &gt; speed / 3.14&quot; , temp : 7 , speed : 3.14 ) . formula &quot;7 - foo * 4 &gt; 3.14 / 3.14&quot;","ref":"Formulae.html#curry/3","title":"Formulae.curry/3","type":"function"},{"doc":"Checks whether the formula was already compiled into module. Similar to compiled?/1 , but returns what Code.ensure_compiled/1 returns. Typically one does not need to call this function, since this check would be nevertheless transparently performed before the evaluation. Examples: iex&gt; Formulae . ensure_compiled ( &quot;bar &gt; 42&quot; ) { :error , :nofile } iex&gt; Formulae . compile ( &quot;bar &gt; 42&quot; ) iex&gt; Formulae . ensure_compiled ( &quot;bar &gt; 42&quot; ) { :module , :&quot;Elixir.Formulae.bar &gt; 42&quot; }","ref":"Formulae.html#ensure_compiled/2","title":"Formulae.ensure_compiled/2","type":"function"},{"doc":"Evaluates the formula returning the result back. Examples: iex&gt; Formulae . eval ( &quot;rem(a, 5) + rem(b, 4) == 0&quot; , a : 20 , b : 20 ) true iex&gt; Formulae . eval ( &quot;rem(a, 5) == 0&quot; , a : 21 ) false iex&gt; Formulae . eval ( &quot;rem(a, 5) + rem(b, 4)&quot; , a : 21 , b : 22 ) 3","ref":"Formulae.html#eval/2","title":"Formulae.eval/2","type":"function"},{"doc":"Evaluates the formula returning the result back; throws in a case of unseccessful processing. Examples: iex&gt; Formulae . eval! ( &quot;rem(a, 5) == 0&quot; , a : 20 ) true iex&gt; Formulae . eval! ( &quot;rem(a, 5) == 0&quot; ) ** (Formulae.RunnerError) Formula failed to run (compile): [:missing_arguments] wrong or incomplete eval call: [given_keys: [], expected_keys: [:a]].","ref":"Formulae.html#eval!/2","title":"Formulae.eval!/2","type":"function"},{"doc":"Evaluates normalized representation of formula. Examples iex&gt; Formulae . eval ( &quot;3 &gt; 2&quot; ) true iex&gt; Formulae . eval ( &quot;3 &lt; 2&quot; ) false iex&gt; Formulae . eval ( &quot;a &lt; 2&quot; , a : 1 ) true iex&gt; Formulae . eval ( &quot;a &gt; 2&quot; , a : 1 ) false iex&gt; Formulae . eval ( &quot;a &lt; 2&quot; , [ ] ) { :error , { :missing_arguments , [ given_keys : [ ] , expected_keys : [ :a ] ] } } iex&gt; Formulae . eval! ( &quot;a &lt; 2&quot; , [ ] ) ** (Formulae.RunnerError) Formula failed to run (compile): [:missing_arguments] wrong or incomplete eval call: [given_keys: [], expected_keys: [:a]]. iex&gt; Formulae . eval ( &quot;a + 2 == 3&quot; , a : 1 ) true iex&gt; Formulae . eval ( &quot;a + 2 == 3&quot; , a : 2 ) false iex&gt; Formulae . eval ( ~S|a == &quot;3&quot;| , a : &quot;3&quot; ) true iex&gt; Formulae . eval ( ~S|a == &quot;3&quot;| , a : 3 ) false iex&gt; Formulae . eval ( ~S|a == &quot;3&quot;| , a : &quot;hello&quot; ) false iex&gt; Formulae . eval ( &quot;a + 2 == 3&quot; , a : 2 ) false iex&gt; Formulae . eval ( ~S|a == &quot;3&quot;| , a : &quot;3&quot; ) true iex&gt; Formulae . eval ( &quot;a_b_c_490000 &gt; 2&quot; , a_b_c_490000 : 3 ) true","ref":"Formulae.html#evaluate/3","title":"Formulae.evaluate/3","type":"function"},{"doc":"Returns a normalized representation for the formula given.","ref":"Formulae.html#normalize/1","title":"Formulae.normalize/1","type":"function"},{"doc":"Generated clauses for n ∈ [1..12] to be used with dynamic number","ref":"Formulae.html#permutations/2","title":"Formulae.permutations/2","type":"function"},{"doc":"Purges and discards the module for the formula given (if exists.)","ref":"Formulae.html#purge/2","title":"Formulae.purge/2","type":"function"},{"doc":"Produces the normalized representation of formula. If the rho is an instance of Integer or Float , it’s left intact, otherwise it’s moved to the left side with negation. Examples iex &gt; Formulae . unit ( &quot;3 &gt; 2&quot; ) { &quot;3 &gt; 2&quot; , { :&gt; , [ ] , [ 3 , 2 ] } } iex &gt; Formulae . unit ( &quot;3 - a &gt; 2&quot; ) { &quot;3 - a &gt; 2&quot; , { :&gt; , [ ] , [ { :- , [ line : 1 ] , [ 3 , { :a , [ line : 1 ] , nil } ] } , 2 ] } } iex &gt; Formulae . unit ( &quot;3 &gt; A + 2&quot; ) { &quot;3 &gt; a + 2&quot; , { :&gt; , [ ] , [ { :- , [ context : Formulae , import : Kernel ] , [ 3 , { :+ , [ line : 1 ] , [ { :a , [ line : 1 ] , nil } , 2 ] } ] } , 0 ] } } iex &gt; Formulae . unit ( &quot;3 &gt;= a + 2&quot; ) { &quot;3 &gt;= a + 2&quot; , { :&gt;= , [ ] , [ { :- , [ context : Formulae , import : Kernel ] , [ 3 , { :+ , [ line : 1 ] , [ { :a , [ line : 1 ] , nil } , 2 ] } ] } , 0 ] } } iex &gt; Formulae . unit ( &quot;3 a &gt; A + 2&quot; ) ** (Formulae.SyntaxError) Formula [3 a &gt; A + 2] syntax is incorrect (parsing): syntax error before: “a”. iex &gt; Formulae . unit ( &quot;a + 2 = 3&quot; ) { &quot;a + 2 = 3&quot; , { :== , [ ] , [ { :+ , [ line : 1 ] , [ { :a , [ line : 1 ] , nil } , 2 ] } , 3 ] } } iex &gt; Formulae . unit ( ~S|A = &quot;3&quot;| ) { &quot;a = \\&quot; 3 \\&quot; &quot; , { :== , [ ] , [ { :a , [ line : 1 ] , nil } , &quot;3&quot; ] } }","ref":"Formulae.html#unit/2","title":"Formulae.unit/2","type":"function"},{"doc":"The formulae is internally represented as struct, exposing the original binary representing the formula, AST, the module this formula was compiled into, variables (bindings) this formula has and the evaluator, which is the function of arity one, accepting the bindings as a keyword list and returning the result of this formula application.","ref":"Formulae.html#t:t/0","title":"Formulae.t/0","type":"type"},{"doc":"Functions to calculate all the combinations, permutations and repeated permutations of the list given. Examples iex&gt; Formulae.Combinators . combinations ( ~w|a b c d|a , 2 ) [ [ :a , :b ] , [ :a , :c ] , [ :a , :d ] , [ :b , :c ] , [ :b , :d ] , [ :c , :d ] ] iex&gt; Formulae.Combinators . permutations ( ~w|a b c d|a , 2 ) [ [ :a , :b ] , [ :a , :c ] , [ :a , :d ] , [ :b , :a ] , [ :b , :c ] , [ :b , :d ] , [ :c , :a ] , [ :c , :b ] , [ :c , :d ] , [ :d , :a ] , [ :d , :b ] , [ :d , :c ] ] iex&gt; Formulae.Combinators . repeated_permutations ( ~w|a b c d|a , 2 ) [ [ :a , :a ] , [ :a , :b ] , [ :a , :c ] , [ :a , :d ] , [ :b , :a ] , [ :b , :b ] , [ :b , :c ] , [ :b , :d ] , [ :c , :a ] , [ :c , :b ] , [ :c , :c ] , [ :c , :d ] , [ :d , :a ] , [ :d , :b ] , [ :d , :c ] , [ :d , :d ] ] NB this functions should not be used for relatively big n because they perform greedy evaluation. See to Formulae.Combinators.Stream for lazy analogues returning streams. Dynamic numbers To use combinations and permutations with dynamic number n , use Formulae.combinations/2 and Formulae.permutations/2 , generated in the compile time. By default, there are 42 clauses for combinations and 12 for permutations are generated. They might be changed in config , keys :formulae, :max_combinations and :formulae, :max_permutations . To suppress generation, use :formulae, :generate_combinators, false . iex&gt; with n &lt;- 2 , do : Formulae . combinations ( ~w|a b c d|a , n ) [ [ :a , :b ] , [ :a , :c ] , [ :a , :d ] , [ :b , :c ] , [ :b , :d ] , [ :c , :d ] ]","ref":"Formulae.Combinators.html","title":"Formulae.Combinators","type":"module"},{"doc":"Calculates all combinations of the list, given as the first parameter","ref":"Formulae.Combinators.html#combinations/2","title":"Formulae.Combinators.combinations/2","type":"macro"},{"doc":"Calculates all permutations of the list, given as the first parameter","ref":"Formulae.Combinators.html#permutations/2","title":"Formulae.Combinators.permutations/2","type":"macro"},{"doc":"Calculates all repeated permutations of the list, given as the first parameter","ref":"Formulae.Combinators.html#repeated_permutations/2","title":"Formulae.Combinators.repeated_permutations/2","type":"macro"},{"doc":"Helper to calculate all the combinations / permutations of the enumerable given. Similar to Formulae.Combinators but returns a stream. Examples iex&gt; ~w|a b c d|a ...&gt; |&gt; Formulae.Combinators.Stream . combinations ( 2 ) ...&gt; |&gt; elem ( 0 ) ...&gt; |&gt; Enum . to_list ( ) [ [ :a , :b ] , [ :a , :c ] , [ :a , :d ] , [ :b , :c ] , [ :b , :d ] , [ :c , :d ] ] iex&gt; ~w|a b c d|a ...&gt; |&gt; Formulae.Combinators.Stream . permutations ( 2 ) ...&gt; |&gt; elem ( 0 ) ...&gt; |&gt; Enum . to_list ( ) [ [ :a , :b ] , [ :a , :c ] , [ :a , :d ] , [ :b , :a ] , [ :b , :c ] , [ :b , :d ] , [ :c , :a ] , [ :c , :b ] , [ :c , :d ] , [ :d , :a ] , [ :d , :b ] , [ :d , :c ] ] iex&gt; ~w|a b c d|a ...&gt; |&gt; Formulae.Combinators.Stream . repeated_permutations ( 2 ) ...&gt; |&gt; elem ( 0 ) ...&gt; |&gt; Enum . to_list ( ) [ [ :a , :a ] , [ :a , :b ] , [ :a , :c ] , [ :a , :d ] , [ :b , :a ] , [ :b , :b ] , [ :b , :c ] , [ :b , :d ] , [ :c , :a ] , [ :c , :b ] , [ :c , :c ] , [ :c , :d ] , [ :d , :a ] , [ :d , :b ] , [ :d , :c ] , [ :d , :d ] ] iex&gt; ( for c &lt;- ?a .. ?z , do : &lt;&lt; c &gt;&gt; ) ...&gt; |&gt; Formulae.Combinators.Stream . combinations ( 12 ) ...&gt; |&gt; elem ( 0 ) ...&gt; |&gt; Stream . take_every ( 26 ) ...&gt; |&gt; Enum . take ( 2 ) [ [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; , &quot;d&quot; , &quot;e&quot; , &quot;f&quot; , &quot;g&quot; , &quot;h&quot; , &quot;i&quot; , &quot;j&quot; , &quot;k&quot; , &quot;l&quot; ] , [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; , &quot;d&quot; , &quot;e&quot; , &quot;f&quot; , &quot;g&quot; , &quot;h&quot; , &quot;i&quot; , &quot;j&quot; , &quot;l&quot; , &quot;x&quot; ] ] iex&gt; l = for c &lt;- ?a .. ?z , do : &lt;&lt; c &gt;&gt; iex&gt; with { stream , :ok } &lt;- Formulae.Combinators.Stream . permutations ( l , 12 ) , ...&gt; do : stream |&gt; Stream . take_every ( 26 ) |&gt; Enum . take ( 2 ) [ [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; , &quot;d&quot; , &quot;e&quot; , &quot;f&quot; , &quot;g&quot; , &quot;h&quot; , &quot;i&quot; , &quot;j&quot; , &quot;k&quot; , &quot;l&quot; ] , [ &quot;a&quot; , &quot;b&quot; , &quot;c&quot; , &quot;d&quot; , &quot;e&quot; , &quot;f&quot; , &quot;g&quot; , &quot;h&quot; , &quot;i&quot; , &quot;j&quot; , &quot;l&quot; , &quot;w&quot; ] ] iex&gt; ( for c &lt;- ?a .. ?z , do : &lt;&lt; c &gt;&gt; ) ...&gt; |&gt; Formulae.Combinators.Stream . repeated_permutations ( 12 ) ...&gt; |&gt; elem ( 0 ) ...&gt; |&gt; Enum . take ( 5 ) [ [ &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; ] , [ &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;b&quot; ] , [ &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;c&quot; ] , [ &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;d&quot; ] , [ &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;a&quot; , &quot;e&quot; ] ]","ref":"Formulae.Combinators.Stream.html","title":"Formulae.Combinators.Stream","type":"module"},{"doc":"Lazily calculates combinations of the list, given as the first parameter, returns {%Stream{}, :ok} tuple. See Formulae.Combinators.combinations/2 for greedy version.","ref":"Formulae.Combinators.Stream.html#combinations/2","title":"Formulae.Combinators.Stream.combinations/2","type":"macro"},{"doc":"Lazily calculates permutations of the list, given as the first parameter, returns {%Stream{}, :ok} tuple. See Formulae.Combinators.permutations/2 for greedy version.","ref":"Formulae.Combinators.Stream.html#permutations/2","title":"Formulae.Combinators.Stream.permutations/2","type":"macro"},{"doc":"Lazily calculates repeated permutations of the list, given as the first parameter, returns {%Stream{}, :ok} tuple. See Formulae.Combinators.repeated_permutations/2 for greedy version.","ref":"Formulae.Combinators.Stream.html#repeated_permutations/2","title":"Formulae.Combinators.Stream.repeated_permutations/2","type":"macro"},{"doc":"Handles the sigil ~F for Formulae . It returns a compiled Formulae instance. Examples iex&gt; import Formulae.Sigils iex&gt; ~F[x / y &gt; 42] % Formulae { ast : Code . string_to_quoted! ( &quot;x / y &gt; 42&quot; ) , eval : &amp; :&quot;Elixir.Formulae.x ÷ y &gt; 42&quot; . eval / 1 , formula : &quot;x / y &gt; 42&quot; , guard : nil , module : :&quot;Elixir.Formulae.x ÷ y &gt; 42&quot; , variables : [ :x , :y ] } iex&gt; ~F[x == y]g % Formulae { ast : Code . string_to_quoted! ( &quot;x == y&quot; ) , eval : &amp; :&quot;Elixir.Formulae.x == y&quot; . eval / 1 , formula : &quot;x == y&quot; , guard : { :defguard , [ { :generated , true } , { :context , Formulae } , { :import , Kernel } ] , [ { :when , [ generated : true ] , [ { :guard , [ generated : true ] , [ { :x , [ ] , nil } , { :y , [ ] , nil } ] } , { :== , [ line : 1 ] , [ { :x , [ line : 1 ] , nil } , { :y , [ line : 1 ] , nil } ] } ] } ] } , module : :&quot;Elixir.Formulae.x == y&quot; , variables : [ :x , :y ] }","ref":"Formulae.Sigils.html","title":"Formulae.Sigils","type":"module"},{"doc":"","ref":"Formulae.RunnerError.html","title":"Formulae.RunnerError","type":"exception"},{"doc":"","ref":"Formulae.SyntaxError.html","title":"Formulae.SyntaxError","type":"exception"},{"doc":"Nifty precompiled formulae handling","ref":"readme.html","title":"Formulae","type":"extras"},{"doc":"def deps do [ { :formulae , &quot;~&gt; 0.5&quot; } ] end","ref":"readme.html#installation","title":"Formulae - Installation","type":"extras"},{"doc":"iex | 1 ▸ f = Formulae . compile ( &quot;rem(a, 5) - b == 0&quot; ) #ℱ&lt;[ ast : &quot;rem(a, 5) - b == 0&quot; , eval : &amp; :&quot;Elixir.Formulae.rem(a, 5) - b == 0&quot; . eval / 1 , formula : &quot;rem(a, 5) - b == 0&quot; , guard : nil , module : :&quot;Elixir.Formulae.rem(a, 5) - b == 0&quot; , variables : [ :a , :b ] ] &gt; iex | 2 ▸ f . eval . ( a : 11 , b : 1 ) true iex | 3 ▸ f . variables [ :a , :b ] iex | 4 ▸ f . module :&quot;Elixir.Formulae.rem(a, 5) - b == 0&quot;","ref":"readme.html#using","title":"Formulae - Using","type":"extras"},{"doc":"0.10.0 Inspect and String.Chars protocols implementation Formulae.Sigils.sigil_F/2 aka ~F[x &gt; y] 0.9.0 options: [eval: :guard, alias: SomeModName] fixes for modern Elixir optimizations in module generation benchmarks 0.8.0 — generate combinators functions in Formulae module","ref":"readme.html#changelog","title":"Formulae - Changelog","type":"extras"},{"doc":"","ref":"readme.html#documentation","title":"Formulae - Documentation","type":"extras"}]